@@ -6,6 +6,10 @@
 #'
 #' @return A matrix of regression coefficients
 #'
+#' @note
+#' The matrix of coefficients can be accessed by removing the class
+#' from the returned object (e.g., \code{unclass(coefs)}).
+#'
 #' @examples
 #'
 #' \donttest{
@@ -24,7 +28,6 @@
 #' }
 #' @export
 coef.ggmncv <- function(object, ...){
-
   Theta <- object$Theta
   coefs <- coef_helper(Theta)
   class(coefs) <- c("ggmncv", "coef")
@@ -34,12 +37,12 @@ coef.ggmncv <- function(object, ...){
 
 print_coef <- function(x,...){
 
-  p <- ncol(x)
+  p <- nrow(x)
   cat("Estimates:\n\n")
   for(i in 1:p){
     cat(paste("node", i, "\n\n"))
-    nodes_id <-  (1:20)[-i]
-    dat <-  as.data.frame(t(cs[i,]))
+    nodes_id <-  (1:p)[-i]
+    dat <-  as.data.frame(t(x[i,]))
     colnames( dat) <- paste("node", nodes_id)
     print(dat, row.names = FALSE  )
     cat("---\n")
